"use strict";

var _require = require("./invoke"),
    invoke = _require.invoke;

var cmds = require("../cmds");

var _require2 = require("assert"),
    deepEqual = _require2.deepEqual;

var _require3 = require("effects-as-data"),
    call = _require3.call;

var _require4 = require("util"),
    promisify = _require4.promisify;

var fs = require("fs");
var readFile = promisify(fs.readFile);
var writeFile = promisify(fs.writeFile);

var obj = {
  a1: function a1() {
    return this.a2();
  },
  a2: function a2() {
    return Promise.resolve("a");
  },
  b1: function b1(done) {
    this.b2(done);
  },
  b2: function b2(done) {
    done(null, "b");
  },
  c1: function c1(done) {
    done(new Error("oops"));
  }
};

test("invoke() should execute a function", async function () {
  var _marked = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  var expected = "foo";
  function test() {
    return regeneratorRuntime.wrap(function test$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return cmds.invoke(writeFile, "/tmp/test-" + now + ".js", expected, {
              encoding: "utf8"
            });

          case 2:
            _context.next = 4;
            return cmds.invoke(readFile, "/tmp/test-" + now + ".js", {
              encoding: "utf8"
            });

          case 4:
            return _context.abrupt("return", _context.sent);

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _marked[0], this);
  }

  var actual = await call({}, { invoke: invoke }, test);
  expect(actual).toEqual(expected);
});

test("invokeCtx() should execute a function with a ctx", async function () {
  var _marked2 = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  function test() {
    return regeneratorRuntime.wrap(function test$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return cmds.invokeCtx(obj.a1, obj);

          case 2:
            return _context2.abrupt("return", _context2.sent);

          case 3:
          case "end":
            return _context2.stop();
        }
      }
    }, _marked2[0], this);
  }

  var actual = await call({}, { invoke: invoke }, test);
  expect(actual).toEqual("a");
});

test("invokeCallback() should execute a callback", async function () {
  var _marked3 = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  var expected = "foo";
  function test() {
    return regeneratorRuntime.wrap(function test$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return cmds.invokeCallback(writeFile, "/tmp/test-" + now + ".js", expected, {
              encoding: "utf8"
            });

          case 2:
            _context3.next = 4;
            return cmds.invokeCallback(readFile, "/tmp/test-" + now + ".js", {
              encoding: "utf8"
            });

          case 4:
            return _context3.abrupt("return", _context3.sent);

          case 5:
          case "end":
            return _context3.stop();
        }
      }
    }, _marked3[0], this);
  }

  var actual = await call({}, { invoke: invoke }, test);
  expect(actual).toEqual(expected);
});

test("invokeNode() should execute a callback", async function () {
  var _marked4 = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  var expected = "foo";
  function test() {
    return regeneratorRuntime.wrap(function test$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            _context4.next = 2;
            return cmds.invokeCallback(writeFile, "/tmp/test-" + now + ".js", expected, {
              encoding: "utf8"
            });

          case 2:
            _context4.next = 4;
            return cmds.invokeNode(readFile, "/tmp/test-" + now + ".js", {
              encoding: "utf8"
            });

          case 4:
            return _context4.abrupt("return", _context4.sent);

          case 5:
          case "end":
            return _context4.stop();
        }
      }
    }, _marked4[0], this);
  }

  var actual = await call({}, { invoke: invoke }, test);
  expect(actual).toEqual(expected);
});

test("invokeCallbackCtx() should execute a function with a ctx and callback", async function () {
  var _marked5 = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  function test() {
    return regeneratorRuntime.wrap(function test$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            _context5.next = 2;
            return cmds.invokeCallbackCtx(obj.b1, obj);

          case 2:
            return _context5.abrupt("return", _context5.sent);

          case 3:
          case "end":
            return _context5.stop();
        }
      }
    }, _marked5[0], this);
  }

  var actual = await call({}, { invoke: invoke }, test);
  expect(actual).toEqual("b");
});

test("invokeNodeCtx() should execute a function with a ctx and callback", async function () {
  var _marked6 = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  function test() {
    return regeneratorRuntime.wrap(function test$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            _context6.next = 2;
            return cmds.invokeNodeCtx(obj.b1, obj);

          case 2:
            return _context6.abrupt("return", _context6.sent);

          case 3:
          case "end":
            return _context6.stop();
        }
      }
    }, _marked6[0], this);
  }

  var actual = await call({}, { invoke: invoke }, test);
  expect(actual).toEqual("b");
});

test("invokeCallback() should be able to handle errors", async function () {
  var _marked7 = [test].map(regeneratorRuntime.mark);

  var now = Date.now();
  function test() {
    return regeneratorRuntime.wrap(function test$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            _context7.next = 2;
            return cmds.invokeCallback(obj.c1);

          case 2:
            return _context7.abrupt("return", _context7.sent);

          case 3:
          case "end":
            return _context7.stop();
        }
      }
    }, _marked7[0], this);
  }

  try {
    await call({}, { invoke: invoke }, test);
  } catch (e) {
    expect(e.message).toEqual("oops");
    return;
  }
  fail("Did not throw");
});
//# sourceMappingURL=invoke.spec.js.map